package com.ideabus.sdk_test;

import androidx.appcompat.app.AppCompatActivity;
import android.os.Bundle;
import androidx.appcompat.widget.Toolbar;
import android.widget.ListView;
import android.widget.Toast;

import com.ideabus.model.bluetooth.MyBluetoothLE;
import com.ideabus.model.data.Oxygen;
import com.ideabus.model.protocol.OxygenProtocol;

public class SPO2TestActivity extends AppCompatActivity implements OxygenProtocol.OnConnectStateListener, OxygenProtocol.OnNotifyStateListener,
        OxygenProtocol.OnDataResponseListener,  MyBluetoothLE.OnWriteStateListener {

    private ListView btListView;
    private Toolbar toolbar;
    private LogListAdapter logListAdapter;
    private boolean isConnecting;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        //Initialize the body ester machine Bluetooth module

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_bt_test);

        initView();
        initParam();
    }


    /**
     * 初始化View
     */
    private void initView() {
        toolbar = (Toolbar) findViewById(R.id.tool_bar);
        btListView = (ListView) findViewById(R.id.bt_list_view);

    }

    /**
     * 初始化
     */
    private void initParam() {
        setSupportActionBar(toolbar);

        //Initialize the connection SDK
        Global.oxygenProtocol = OxygenProtocol.getInstance(this, false, true, Global.sdkid_SPO2);
        toolbar.setSubtitle("Oxygen " + Global.oxygenProtocol.getSDKVersion());

        logListAdapter = new LogListAdapter(this);
        btListView.setAdapter(logListAdapter);
    }

    @Override
    protected void onStart() {
        super.onStart();
        Global.oxygenProtocol.setOnDataResponseListener(this);
        Global.oxygenProtocol.setOnConnectStateListener(this);
        Global.oxygenProtocol.setOnNotifyStateListener(this);
        Global.oxygenProtocol.setOnWriteStateListener(this);

        //Start scan
        startScan();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (Global.oxygenProtocol.isConnected()) Global.oxygenProtocol.disconnect();
        Global.oxygenProtocol.stopScan();
    }

    private void startScan() {
        if (!Global.oxygenProtocol.isSupportBluetooth(this)) {
            return;
        }
        logListAdapter.addLog("start scan");
        Global.oxygenProtocol.startScan(10);
    }

    @Override
    protected void onStop() {
        super.onStop();
        Global.oxygenProtocol.disconnect();
        Global.oxygenProtocol.stopScan();
    }

    @Override
    public void onBtStateChanged(boolean isEnable) {
        //BLE will be returned when it is turned enable or disable
        if (isEnable) {
            Toast.makeText(this, "BLE is enable!!", Toast.LENGTH_SHORT).show();
            startScan();
        } else
            Toast.makeText(this, "BLE is disable!!", Toast.LENGTH_SHORT).show();
    }

    @Override
    public void onScanResult(String mac, String name, int rssi) {
        //Temperature
        if (!name.startsWith("n/a")) {
            logListAdapter.addLog("onScanResult："+name+" mac:"+mac+" rssi:"+rssi);
        }
        if (isConnecting)
            return;
        isConnecting = true;

        //Stop scanning before connecting
        Global.oxygenProtocol.stopScan();
        //Connection
        Global.oxygenProtocol.connect(mac);
    }

    @Override
    public void onConnectionState(OxygenProtocol.ConnectState connectState) {
        switch (connectState) {
            case Connected:
                isConnecting = false;
                logListAdapter.addLog("Connected");
                break;
            case ConnectTimeout:
                isConnecting = false;
                logListAdapter.addLog("ConnectTimeout");
                break;
            case Disconnect:
                isConnecting = false;
                logListAdapter.addLog("Disconnected");
                startScan();
                break;
            case ScanFinish:
                logListAdapter.addLog("ScanFinish");
                startScan();
                break;
        }
    }


    @Override
    public void onWriteMessage(boolean isSuccess, String message) {
        logListAdapter.addLog("WRITE : " + message);
    }

    @Override
    public void onNotifyMessage(String message) {
        logListAdapter.addLog("NOTIFY : " + message);
    }

    @Override
    public void onResponseReadOxygenData(Oxygen oxygen) {
        logListAdapter.addLog("Oxygen : onResponseReadOxygenData -> OxygenMeasureData = " + oxygen);
    }
}
